/* execlass.cc
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2002 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: execlass.cc,v 1.4 2002/08/24 01:53:17 ralf Exp $ */

#include "execlass.h"
#include <stdarg.h>
#include "datei.h"
#include <aguix/lowlevelfunc.h>
#include "locale.h"
#include "wconfig.h"
#include "worker.h"

ExeClass::ExeClass( execlass_execute_t exetype, Worker *w )
{
  worker = w;
  type = exetype;
  tmpfile = Datei::createTMPName();
  tmpoutput = Datei::createTMPName();
  tmperror = Datei::createTMPName();
  fp = fopen( tmpoutput, "w" );
  if ( fp != NULL ) {
    /* be sure the output can only be changed be use */
    chmod( tmpoutput, S_IRUSR | S_IWUSR | S_IXUSR );
    fclose( fp );
  }
  fp = fopen( tmperror, "w" );
  if ( fp != NULL ) {
    /* be sure the output can only be changed be use */
    chmod( tmperror, S_IRUSR | S_IWUSR );
    fclose( fp );
  }
  fp = fopen( tmpfile, "w" );
  if ( fp != NULL ) {
    chmod( tmpfile, S_IRUSR | S_IWUSR | S_IXUSR );
    fprintf( fp, "#! /bin/sh\n\n" );
  }
  lasterror = 0;
}

ExeClass::~ExeClass()
{
  if ( fp != NULL ) {
    fclose( fp );
  }
  remove( tmpfile );
  _freesafe( tmpfile );
  remove( tmpoutput );
  _freesafe( tmpoutput );
  remove( tmperror );
  _freesafe( tmperror );
}

void ExeClass::addCommand( const char *format, ... )
{
  va_list va;
  int n;
  
  if ( ( fp != NULL ) && ( format != NULL ) ) {
    va_start( va, format );
    n = vfprintf( fp, format, va );
    va_end( va );
    if ( type == EXECUTE_SHOW_OUTPUT ) {
      fprintf( fp, ">>%s\n", tmpoutput );
    } else {
      fprintf( fp, "\n" );
    }
  }
}

void ExeClass::execute()
{
  char *exestr = NULL;
  
  if ( fp == NULL ) return;

  if ( worker != NULL ) worker->setWaitCursor();

  if ( ( type == EXECUTE_TERMINAL_WAIT ) ||
       ( type == EXECUTE_TERMINAL ) ) {
    if ( type == EXECUTE_TERMINAL_WAIT ) {
      fprintf( fp, "echo %s\n", catalog.getLocale( 201 ) );
      fprintf( fp, "read x\n" );
    }
    exestr = (char*)_allocsafe( strlen( wconfig->getTerminalStr() ) +
                                strlen( tmpfile ) + 1 );
    sprintf( exestr, wconfig->getTerminalStr(), tmpfile );
  } else {
    exestr = dupstring( tmpfile );
  }
  if ( type == EXECUTE_SHOW_OUTPUT ) {
    fprintf( fp, OUTPUT_BIN, tmpoutput );
    fprintf( fp, "\n" );
  }
  fclose( fp );
  fp = NULL;
  system( exestr );
  _freesafe(exestr);

  if ( worker != NULL ) worker->unsetWaitCursor();
}

char *ExeClass::getOutput()
{
  return getOutput( NULL );
}

char *ExeClass::getOutput( int *return_error )
{
  std::string exestr;
  Datei *d;
  char *erg = NULL;
  
  /* only when type is normal because we want no user interaction */
  if ( type != EXECUTE_NORMAL ) return NULL;
  if ( fp == NULL ) return NULL;
  
  if ( worker != NULL ) worker->setWaitCursor();

  fclose( fp );
  fp = NULL;

  exestr = tmpfile;
  exestr += " >";
  exestr += tmpoutput;
  exestr += " 2>";
  exestr += tmperror;
  system( exestr.c_str() );
  d = new Datei();
  if ( d->open( tmpoutput, "r" ) == 0 ) {
    erg = d->getLine();
    d->close();
  }
  delete d;
  if ( Datei::fileExistsExt( tmperror ) == Datei::D_FE_FILE ) {
    if ( Datei::fileSize( tmperror ) != 0 ) {
      // filesize has to be 0 if anything was okay
      lasterror = 2;
    } else {
      lasterror = 0;
    }
  } else {
    // error file not found => assume something gone wrong
    lasterror = 1;
  }
  if ( return_error != NULL ) *return_error = lasterror;

  if ( worker != NULL ) worker->unsetWaitCursor();

  return erg;
}

int ExeClass::getReturnCode()
{
  return getReturnCode( NULL );
}

int ExeClass::getReturnCode( int *return_error )
{
  int erg;
  std::string exestr;
  
  /* only when type is normal because we want no user interaction */
  if ( type != EXECUTE_NORMAL ) return -1;
  if ( fp == NULL ) return -1;
  
  if ( worker != NULL ) worker->setWaitCursor();

  fclose( fp );
  fp = NULL;

  exestr = tmpfile;
  exestr += " 2>";
  exestr += tmperror;

  erg = system( exestr.c_str() );

  if ( Datei::fileExistsExt( tmperror ) == Datei::D_FE_FILE ) {
    if ( Datei::fileSize( tmperror ) != 0 ) {
      // filesize has to be 0 if anything was okay
      lasterror = 2;
    } else {
      lasterror = 0;
    }
  } else {
    // error file not found => assume something gone wrong
    lasterror = 1;
  }
  if ( return_error != NULL ) *return_error = lasterror;

  if ( worker != NULL ) worker->unsetWaitCursor();

  return erg;
}

void ExeClass::cdDir( const char *dir )
{
  if ( ( fp != NULL ) && ( dir != NULL ) ) {
    fprintf( fp, "cd \"%s\"\n", dir );
  }
}

int ExeClass::getLastError()
{
  return lasterror;
}

